<?php

namespace App\Admin\Controllers\OrdersManagement;

use App\Admin\Actions\backstage\Refund\RefundCompleteAction;
use App\Models\Organization;
use App\Models\Patient;
use App\Models\RefundApplication;
use App\Models\User;
use Encore\Admin\Controllers\AdminController;
use Encore\Admin\Form;
use Encore\Admin\Grid;
use Encore\Admin\Show;
use Encore\Admin\Auth\Database\Administrator;
use Encore\Admin\Auth\Permission;

class RefundApplicationController extends AdminController
{
    /**
     * Title for current resource.
     *
     * @var string
     */
    protected $title = '退款申请列表';

    /**
     * Make a grid builder.
     *
     * @return Grid
     */
    protected function grid()
    {
        $grid = new Grid(new RefundApplication());
        $grid->disableCreateButton();
        $grid->disableBatchActions();
        $grid->actions(function ($actions){
           $actions->disableView();
           $actions->disableEdit();
           $actions->disableDelete();
           if ($actions->row->status == 1)
           {
               if (\Admin::user()->isRole('financial_staff')){
                   $actions->add(new RefundCompleteAction());
               }
           }
        });
        $grid->column('id', __('id'));
        $grid->column('order_id', __('订单id'));
        $grid->column('orders.order_sn', __('订单号'));
        $grid->column('orders.user_id', __('用户名'))->display(function ($id){
            $name = User::where('id',$id)->value('nickname');
            return $name;
        });
        $grid->column('orders.patient_id', __('患者名'))->display(function ($id){
            $name = Patient::where('id',$id)->value('name');
            return $name;
        });
        $grid->column('orders.organization_id', __('机构名'))->display(function ($id){
            if ($id == 0 )
            {
                return '';
            }
            else
            {
                $name = Organization::where('id',$id)->value('name');
                return $name;
            }
        });
        $grid->column('orders.payment_amount', __('退款金额'))->display(function ($money){
            return $money/100;
        });
        $grid->column('status', __('退款状态'))->using([1=>'退款中',2=>'退款完成']);
        $grid->column('refund_applicant', __('退款申请人员'))->display(function ($id){
            $name = Administrator::where('id',$id)->value('name');
            return $name;
        });
        $grid->column('financial_auditor', __('财务审核人员'))->display(function ($id){
            $name = Administrator::where('id',$id)->value('name');
            return $name;
        });
        $grid->column('created_at', __('退款发起时间'));
        $grid->column('end_time', __('退款完成时间'))->display(function ($time){
            if ($time == 0)
            {
                return '';
            }
            else
            {
                return $time;
            }
        });
        $grid->column('updated_at', __('更新时间'));

        return $grid;
    }

    /**
     * Make a show builder.
     *
     * @param mixed $id
     * @return Show
     */
    protected function detail($id)
    {
        $show = new Show(RefundApplication::findOrFail($id));

        $show->field('id', __('Id'));
        $show->field('order_id', __('Order id'));
        $show->field('status', __('Status'));
        $show->field('refund_applicant', __('Refund applicant'));
        $show->field('financial_auditor', __('Financial auditor'));
        $show->field('end_time', __('End time'));
        $show->field('created_at', __('Created at'));
        $show->field('updated_at', __('Updated at'));

        return $show;
    }

    /**
     * Make a form builder.
     *
     * @return Form
     */
    protected function form()
    {
        $form = new Form(new RefundApplication());

        $form->number('order_id', __('Order id'));
        $form->switch('status', __('Status'));
        $form->number('refund_applicant', __('Refund applicant'));
        $form->number('financial_auditor', __('Financial auditor'));
        $form->datetime('end_time', __('End time'))->default(date('Y-m-d H:i:s'));

        return $form;
    }
}
